#ifndef _AFFINE_HPP_
#define _AFFINE_HPP_

#include <iostream>

// Exemple de classe generique : les fonctions affines.

template<typename T> class affine ;

template<typename T> affine<T> operator+(affine<T> const&, affine<T> const&);
template<typename T> std::ostream& operator<<(std::ostream&, affine<T> const&);

template<typename T>
class affine
{
private:
  T a, b; // Represente la fonction x -> ax+b
public:
  affine(T a,T b); // Constructeur (fournissant les valeurs de a et b)
  
  // Ici, pas besoin de définir l'operateur =, le constructeur par copie et le destructeur
  // (ils sont correctement definis par defaut, car la classe ne contient pas de champs de type pointeur).

  
  affine operator() (affine const&) const; // Composee de deux fonctions affines
  T operator() (T x) const; // Evaluation d'une fonction en un point
  
  friend affine operator+<T>(affine const&, affine const&);
  affine& operator+=(affine const&);

  friend std::ostream& operator<<<T>(std::ostream&, affine const&);
  
};


// Constructeur
template <typename T>
affine<T>::affine(T a_,T b_) : a(a_), b(b_) {};



// Composee de fonctions affines
template <typename T>
affine<T> affine<T>::operator() (affine<T> const&f) const
{
  return affine<T>(a*f.a, a*f.b + b);
}


// Evaluation en un point
template <typename T>
T affine<T>::operator() (T x) const
{
  return a*x+b;
}



// Somme de deux fonctions
template <typename T>
affine<T> operator+(affine<T> const&f, affine<T> const&g)
{
  return affine<T>(f.a + g.a, f.b + g.b);
}

template <typename T>
affine<T>& affine<T>::operator+=(affine<T> const& f)
{
  *this = *this + f;
  return *this;
}


template <typename T>
std::ostream& operator<<(std::ostream &o, affine<T> const &f)
{
  o << "x->";
  if (f.a==0)
	o << f.b;
  else
	{
	  o << f.a << "x";
	  if (f.b >= 0)
		{
		  o << "+" << f.b;
		}
	  else
		o << f.b;
	}
  
  return o;
}




#endif /* _AFFINE_HPP_ */
